local HealingTracker = _G.HealingTracker

local Segments = HealingTracker.Segments

function Segments:Clear()
	local pdb = HealingTracker.ProfileOptions
	pdb.Segments = {}
	Segments.Current = nil
	Segments:Navigate(0)
end

function Segments:ClearTrash()
    local pdb = HealingTracker.ProfileOptions

	for _ = 1, 13 do
		for i=1,#pdb.Segments do
			local v = pdb.Segments[i]
			if v and v.IsBoss and v.IsBoss ~= 1 then
				tremove(pdb.Segments, i)
			end
		end
	end

	Segments:Navigate(#pdb.Segments)
end

function Segments:ClearCurrent()
	local pdb = HealingTracker.ProfileOptions
	local i = #pdb.Segments
	local cur = pdb.Segments[i]
	if cur and pdb.Segments then
		tremove(pdb.Segments, i)
	end
	Segments:Navigate(#pdb.Segments)
end

function Segments:Begin(title)

	local pdb = HealingTracker.ProfileOptions

    local segment = {
		Title = title,
		Start = date("%H:%M:%S"),
		End = nil,
		Duration = 0,
		DurationString = "00:00",
		IsBoss = 0,
		IsTotal = false,
		IsWipe = false,
		Players = {},
		Healers = {}
	}

	Segments.Current = segment
	Segments.Combat = segment
	pdb.CurrentSegment = 0

	local afflix = nil
	if UnitInRaid("player") then
		afflix = "raid"
	elseif UnitInParty("player") then
		afflix = "party"
	end

	if afflix then
		for j=1, GetNumGroupMembers() do
			if UnitIsPlayer(afflix..j) and UnitGroupRolesAssigned(afflix..j) == "HEALER" then
				local uname = UnitName(afflix..j)
				local uguid = UnitGUID(afflix..j)
				segment.Healers[#segment.Healers + 1] = uname
				local hplayer = Segments:GetCombatPlayer(uname, uguid)
				hplayer.IsHealer = 1
			end
		end
	end

	local myname = UnitName("player")
	local myguid = UnitGUID("player")
	segment.Healers[#segment.Healers + 1] = myname
	local me = Segments:GetCombatPlayer(myname, myguid)
	me.IsHealer = 1

	return segment
end

function Segments:FinishCurrent(save)
	local pdb = HealingTracker.ProfileOptions

	if save == 1 then
		if not Segments.Combat.Title then
			if HealingTracker.CurrentInstance.Data then
				Segments.Combat.Title = HealingTracker.CurrentInstance.Data.Abbreviation..": Trash"
			else
				Segments.Combat.Title = "Trash"
			end
		end

		local inext = #pdb.Segments + 1
		if #pdb.Segments > 19 then
			for i=1, 20 do
				pdb.Segments[i] = pdb.Segments[i+1]
			end
			inext = 20
		end

		Segments.Combat.End = date("%H:%M:%S")
		Segments.Combat.Duration = HealingTracker.Combat.CombatSeconds
		Segments.Combat.DurationString = HealingTracker.Helper:GetTimeSpan(HealingTracker.Combat.CombatSeconds)
		local pdb = HealingTracker.ProfileOptions
		pdb.Segments[inext] = Segments.Combat
		Segments.Combat = nil
		pdb.CurrentSegment = inext
	end

end

function Segments:Navigate(index)

	local pdb = HealingTracker.ProfileOptions
	pdb.CurrentSegment = index
	if index == 0 then
		if pdb.Recording == 1 then
			Segments.Current = Segments.Combat
		else
			Segments.Current = pdb.Segments[#pdb.Segments]
		end
	elseif index < 0 then
		Segments.Current = Segments:CreateTotalSegment()
	else
		Segments.Current = pdb.Segments[index]
	end

	Segments:SortCurrent()
	HealingTracker.TrackerFrame:SetSegment(Segments.Current)

end

function Segments:ShowMenu()

	local pdb = HealingTracker.ProfileOptions
	local segMenu = CreateFrame("Frame", "Combats")

	segMenu.displayMode = "MENU"

	segMenu.initialize = function(self, level)
		UIDropDownMenu_AddButton({text = "Segments", isTitle = 1}, level)

		local total = {
			text = "Total",
			checked = pdb.CurrentSegment < 0,
			func = function() Segments:Navigate(-1) end
		}
		UIDropDownMenu_AddButton(total, level)

		local current = {
			text = "Current",
			checked = pdb.CurrentSegment == 0,
			colorCode = "|cFFFFFF00",
			func = function() Segments:Navigate(0) end
		}

		UIDropDownMenu_AddButton(current, level)

		for i=1,#pdb.Segments do
			local v = pdb.Segments[i]
			local vdur = "00:00"
			if v.DurationString == nil then
				v.DurationString = "00:00"
			else
				vdur = v.DurationString
			end
			if v.Title ~= nil then
				local logBtn = {
					text = v.Title.." - "..vdur,
					checked = pdb.CurrentSegment == i,
					func = function()
						Segments:Navigate(i)
					end
				}
				if v.IsBoss == 1 then
					if v.IsWipe == 1 then
						logBtn.colorCode = "|cFFFF3300"
					else
						logBtn.colorCode = "|cFF00FF44"
					end
				else
					logBtn.colorCode = "|cFFAAAAAA"
				end
				UIDropDownMenu_AddButton(logBtn, level)
			end
		end

	end
	
	local x,y = GetCursorPosition(UIParent);
	ToggleDropDownMenu(1, nil, segMenu, "UIParent", x / UIParent:GetEffectiveScale() , y / UIParent:GetEffectiveScale())
	
end

function Segments:CreateTotalSegment()
	
	local pdb = HealingTracker.ProfileOptions
	
	local totalSegment = {}
	totalSegment.Players = {}
	
	table.foreach(pdb.Segments,
		function(ks,vs)
			
			table.foreach(vs.Players, function(k,v)
				local totalLog = nil

				table.foreach(totalSegment.Players, function(k1, v1)
					if totalLog == nil and v1.Name == v.Name then
						totalLog = v1
					end
				end)
				
				if totalLog == nil then
					totalLog =  {
						Name = v.Name,
						Guid = v.Guid,
						TotalMultiply = 0,
						TotalRegular = 0,
						TotalPoint = 0,
						AidfulHeal = 0,
						PointlessHeal = 0,
						AidfulPeriodicHeal = 0,
						PointlessPeriodicHeal = 0,
						Class = v.Class,
						OverHeal = v.OverHeal,
						Timeline = {},
						Spells = {}
					}
					totalSegment.Players[#totalSegment.Players + 1] = totalLog
				end
				
				totalLog.TotalMultiply = totalLog.TotalMultiply + v.TotalMultiply
				totalLog.TotalRegular = totalLog.TotalRegular + v.TotalRegular
				totalLog.TotalPoint = totalLog.TotalPoint + v.TotalPoint
				totalLog.AidfulHeal = totalLog.AidfulHeal + v.AidfulHeal
				totalLog.PointlessHeal = totalLog.PointlessHeal + v.PointlessHeal
				totalLog.AidfulPeriodicHeal = totalLog.AidfulPeriodicHeal + v.AidfulPeriodicHeal
				totalLog.PointlessPeriodicHeal = totalLog.PointlessPeriodicHeal + v.PointlessPeriodicHeal
				totalLog.OverHeal = totalLog.OverHeal + v.OverHeal
				
				table.foreach(v.Timeline, function(k2,v2)
					totalLog.Timeline[#totalLog.Timeline + 1] = v2
				end)
				
				table.foreach(v.Spells, function(k2, v2)
					local logSpell = nil
					table.foreach(totalLog.Spells, function (k3, v3)
						if logSpell == nil and v3.Name == v2.Name then
							logSpell = v3
						end
					end)
					if logSpell == nil then
						logSpell = {
							Id = v2.Id,
							Name = v2.Name,
							Cast = 0,
							Critical = 0,
							PointlessHeal = 0,
							AidfulHeal = 0,
							Point = 0,
							PointMultiply = 0,
							PointRegular = 0,
							AidfulPeriodicHeal = 0,
							PointlessPeriodicHeal = 0,
							OverHeal = 0
						}
						totalLog.Spells[#totalLog.Spells + 1] = logSpell
					end
					logSpell.Cast = logSpell.Cast + v2.Cast
					logSpell.Critical = logSpell.Critical + v2.Critical
					logSpell.PointlessHeal = logSpell.PointlessHeal + v2.PointlessHeal
					logSpell.AidfulHeal = logSpell.AidfulHeal + v2.AidfulHeal
					logSpell.PointlessPeriodicHeal = logSpell.PointlessPeriodicHeal + v2.PointlessPeriodicHeal
					logSpell.AidfulPeriodicHeal = logSpell.AidfulPeriodicHeal + v2.AidfulPeriodicHeal
					logSpell.Point = logSpell.Point + v2.Point
					logSpell.PointMultiply = logSpell.PointMultiply + v2.PointMultiply
					logSpell.PointRegular = logSpell.PointRegular + v2.PointRegular
					logSpell.OverHeal = logSpell.OverHeal + v2.OverHeal
				end)
			end)
			
		end)
	return totalSegment
end

function Segments:GetCombatPlayer(player, guid)
	
	for i, v in ipairs(Segments.Combat.Players) do
		if v.Guid == guid then return v end
	end

	local newRecord = {
		Timeline = {},
		Spells = {},
		Name = player,
		Guid = guid,
		OverHeal = 0,
		TotalMultiply = 0,
		TotalRegular = 0,
		TotalPoint = 0,
		AidfulHeal = 0,
		PointlessHeal = 0,
		AidfulPeriodicHeal = 0,
		PointlessPeriodicHeal = 0,
		Class = UnitClass(player),
		IsHealer = 0,
		Cooldowns = 0,
		Mana = {},
		HPS = {},
		ActiveHeal = 0,
		LoseMana = 0,
		CurrentMana = 0
	}
	newRecord.Mana[#newRecord.Mana] = 100

	Segments.Combat.Players[#Segments.Combat.Players + 1] = newRecord
	return newRecord

end

function Segments:UpdateCombatHealers(second)
	local pdb = HealingTracker.ProfileOptions
    for i, v in ipairs(Segments.Combat.Healers) do
        if v then
			local guid = UnitGUID(v)
			local healer = Segments:GetCombatPlayer(v, guid)
			local percent = (UnitPower(v, 0) / UnitPowerMax(v)) * 100
			if percent > 100 then percent = 100 end
			if percent < 0 then percent = 0 end
			healer.Mana[second] = percent
            healer.CurrentMana = percent
			if second > pdb.ActiveHealThreshold then
				local removing = healer.HPS[second - pdb.ActiveHealThreshold]
				if removing then
					healer.ActiveHeal = healer.ActiveHeal - removing
				end
            end
			local adding = 0
			if healer.HPS[second] then
				adding = healer.HPS[second]
				healer.ActiveHeal = healer.ActiveHeal + adding
			end
		end
	end
end

function Segments:GetCombatPlayerSpell(playerSegment, spellId, spellName)
	local playerSpell = nil
	table.foreach(playerSegment.Spells,
		function(k,v)
			if spellId == v.Id then
				playerSpell = v
			end
		end)
	if playerSpell == nil then
		playerSpell = {
			Id = spellId,
			Name = spellName,
			Cast = 0,
			Critical = 0,
			PointlessHeal = 0,
			AidfulHeal = 0,
			AidfulPeriodicHeal = 0,
			PointlessPeriodicHeal = 0,
			Point = 0,
			PointMultiply = 0,
			PointRegular = 0,
			OverHeal = 0
		}
		playerSegment.Spells[#playerSegment.Spells + 1] = playerSpell
	end
	return playerSpell
end

function Segments:AddOverHeal(caster, target, casterGuid, spellId, spellName, overheal, scritical)
	local tg = Segments:GetCombatPlayer(caster, casterGuid)
	if tg == nil or tg.Class == nil then return end
	local playerSpell = Segments:GetCombatPlayerSpell(tg, spellId, spellName)
	if overheal and overheal > 0 then
		tg.OverHeal = tg.OverHeal + overheal
		playerSpell.OverHeal = playerSpell.OverHeal + overheal
	end
	if scritical then
		playerSpell.Critical = playerSpell.Critical + 1
	else
		playerSpell.Cast = playerSpell.Cast + 1
	end
end

function Segments:AddPointlessHeal(caster, target, casterGuid, spellId, spellName, amount, heal, scritical, periodic)
	
	local tg = Segments:GetCombatPlayer(caster, casterGuid)
	if tg == nil or tg.Class == nil then return end

	local playerSpell = Segments:GetCombatPlayerSpell(tg, spellId, spellName)

	if tg.HPS then
		if tg.HPS[HealingTracker.Combat.CombatSeconds] then
			tg.HPS[HealingTracker.Combat.CombatSeconds] = tg.HPS[HealingTracker.Combat.CombatSeconds] + heal
		else
			tg.HPS[HealingTracker.Combat.CombatSeconds] = heal
		end
	end

	if periodic then
		tg.PointlessPeriodicHeal = tg.PointlessPeriodicHeal + heal
		playerSpell.PointlessPeriodicHeal = playerSpell.PointlessPeriodicHeal + heal
	else
		tg.PointlessHeal = tg.PointlessHeal + heal
		playerSpell.PointlessHeal = playerSpell.PointlessHeal + heal
	end
	if scritical then
		playerSpell.Critical = playerSpell.Critical + 1
	else
		playerSpell.Cast = playerSpell.Cast + 1
	end

end

function Segments:AddAidfulHeal(caster, target, casterGuid, spellId, spellName, amount, heal, scritical, point, firstPerc, multiplyPoint, sumPoint, periodic)

	local tg = Segments:GetCombatPlayer(caster, casterGuid)
	if tg == nil or tg.Class == nil then return end

	if tg.HPS then
		if tg.HPS[HealingTracker.Combat.CombatSeconds] then
			tg.HPS[HealingTracker.Combat.CombatSeconds] = tg.HPS[HealingTracker.Combat.CombatSeconds] + heal
		else
			tg.HPS[HealingTracker.Combat.CombatSeconds] = heal
		end
	end

	tg.TotalPoint = tg.TotalPoint + point
	tg.TotalMultiply = tg.TotalMultiply + multiplyPoint
	tg.TotalRegular = tg.TotalRegular + sumPoint
	if periodic and periodic == 1 then
		tg.AidfulPeriodicHeal = tg.AidfulPeriodicHeal + heal
	else
		tg.AidfulHeal = tg.AidfulHeal + heal
	end
	tg.Timeline[#tg.Timeline + 1] = {
		Date = HealingTracker.Combat.CombatSeconds,
		Target = target,
		BeforeHP = firstPerc,
		AfterHP = HealingTracker.Helper:GUP(target),
		Heal = heal,
		SpellId = spellId,
		SpellName = spellName,
		IsCritical = scritical,
		Point = point,
		MultiplyPoint = multiplyPoint,
		SumPoint = sumPoint,
		IsPeriodic = periodic,
		IsCooldown = nil
	}
	local playerSpell = Segments:GetCombatPlayerSpell(tg, spellId, spellName)
	if periodic and periodic == 1 then
		playerSpell.AidfulPeriodicHeal = playerSpell.AidfulPeriodicHeal + heal
	else
		playerSpell.AidfulHeal = playerSpell.AidfulHeal + heal
	end
	playerSpell.Point = playerSpell.Point + point
	playerSpell.PointMultiply = playerSpell.PointMultiply + multiplyPoint
	playerSpell.PointRegular = playerSpell.PointRegular + sumPoint
	if scritical then
		playerSpell.Critical = playerSpell.Critical + 1
	else
		playerSpell.Cast = playerSpell.Cast + 1
	end

end

function Segments:AddCooldown(spellId, caster, target, casterGuid, targetGuid, spellName, hasTarget)
	local tg = Segments:GetCombatPlayer(caster, casterGuid)
	if tg == nil or tg.Class == nil then return end
	local ftarget = nil
	if hasTarget and hasTarget == 1 then
		ftarget = target
	end
	tg.Cooldowns = tg.Cooldowns + 1
	tg.Timeline[#tg.Timeline + 1] = {
		Date = HealingTracker.Combat.CombatSeconds,
		Target = ftarget,
		BeforeHP = 0,
		AfterHP = 0,
		Heal = 0,
		SpellId = spellId,
		SpellName = spellName,
		IsCritical = nil,
		Point = 0,
		MultiplyPoint = 0,
		SumPoint = 0,
		IsPeriodic = nil,
		IsCooldown = 1
	}
	HealingTracker.TrackerFrame:UpdateBars()
end

function Segments:SortCurrent()
	local helper = HealingTracker.Helper
	local selectedSegment = Segments.Current
	local pdb = HealingTracker.ProfileOptions

	if selectedSegment == nil then
		selectedSegment = {
			EncounterId = 0,
			Title = title,
			Start = date("%H:%M:%S"),
			End = nil,
			Duration = 0,
			DurationString = "00:00",
			IsBoss = 0,
			IsTotal = false,
			IsWipe = 0,
			Players = {}
		}
	end

	if selectedSegment.IsTotal == true then
		selectedSegment = Segments:CreateTotalSegment()
	end
	local slogs = selectedSegment.Players
	for i=#slogs,2,-1 do
		local cur = slogs[i]
		local prev = slogs[i - 1]

		if cur ~= nil and prev ~= nil then
			local cv = false
			if pdb.Columns.Right == 1 then
				cv = helper:TrackPointTableSort(prev, cur)
			elseif pdb.Columns.Right == 2 then
				cv = helper:TotalHealTableSort(prev, cur)
			elseif pdb.Columns.Right == 3 then
				cv = helper:AidfulHealTableSort(prev, cur)
			elseif pdb.Columns.Right == 4 then
				cv = helper:PointlessHealTableSort(prev, cur)
			elseif pdb.Columns.Right == 5 then
				if prev.CurrentMana and cur.CurrentMana then
					cv = prev.CurrentMana < cur.CurrentMana
				elseif prev.CurrentMana then
					cv = true
				else
					cv = false
				end
			elseif pdb.Columns.Right == 6 then
				if prev.LoseMana and cur.LoseMana then
					cv = prev.LoseMana < cur.LoseMana
				elseif prev.LoseMana then
					cv = true
				else
					cv = false
				end
			elseif pdb.Columns.Right == 7 then
				cv = helper:ActiveHealTableSort(prev, cur)
			end

			if cv == true then
				slogs[i] = prev
				slogs[i - 1] = cur
			end
		else
			break
		end
	end

	return selectedSegment
 end