local HealingTracker = _G.HealingTracker
local TrackerFrame = HealingTracker.TrackerFrame
local window = nil
local titleBar = nil
local pdb = nil
local tbar = nil
local windowBars = {}

local LSM = LibStub("LibSharedMedia-3.0")

function TrackerFrame:Init()

	pdb = HealingTracker.ProfileOptions
    window = CreateFrame("Frame", nil, UIParent)

	window:RegisterForDrag("LeftButton")
	window:EnableMouse(true)
	window:SetMovable(true)
	window:SetResizable(true)
	window:SetMinResize(120,80)
	window:SetMaxResize(400,800)
	window:SetFrameStrata("BACKGROUND")
	window:SetSize(pdb.Bounds.Width, pdb.Bounds.Height)

	titleBar = CreateFrame("Frame", "titleBar", window)
	titleBar:SetSize(window:GetWidth(), 21)
    titleBar:SetPoint("TOPLEFT", window, "TOPLEFT",0,0)

    titleBar.Backdrop = CreateFrame("Frame", "titleBarBackdrop", titleBar, BackdropTemplateMixin and "BackdropTemplate")
    titleBar.Backdrop:SetAllPoints()
    titleBar.Backdrop.backdropInfo = {
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark",
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        tile = true,
        tileSize = 16,
        edgeSize = 1,
        insets = {left = 1, right = 1, top = 1, bottom = 1},
    }
    titleBar.Backdrop:ApplyBackdrop()
    titleBar.Backdrop:SetBackdropColor(0, 0, 0, 0.1)

	window:SetScript("OnSizeChanged", function(self)
						if self.isResizing then
							titleBar:SetWidth(self:GetWidth())
							window.bodyFrame:SetHeight(self:GetHeight() - 20)
							window.bodyFrame:SetWidth(self:GetWidth())
							window.bodyScrollFrame:SetWidth(self:GetWidth())
							TrackerFrame:ApplyResizing()
						end
					end)

	window:SetPoint(pdb.Bounds.Point, UIParent, pdb.Bounds.RelativePoint, pdb.Bounds.XOff, pdb.Bounds.YOff)
    window.Backdrop = CreateFrame("Frame", "windowBackdrop", window, "BackdropTemplate")
    window.Backdrop:SetAllPoints()
    window.Backdrop.backdropInfo = {
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark",
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        tile = true,
        tileSize = 16,
        edgeSize = 1,
        insets = {left = 1, right = 1, top = 1, bottom = 1},
    }
    window.Backdrop:ApplyBackdrop()
    window.Backdrop:SetBackdropColor(0, 0, 0, 0.5)

	local report = CreateFrame("Button", nil, titleBar)
	report:SetNormalTexture("Interface\\AddOns\\HealingTracker\\textures\\report")
	report:SetHighlightTexture("Interface\\AddOns\\HealingTracker\\textures\\report", "ADD")
	report:SetSize(12, 12)
	report:SetPoint("TOPRIGHT", window, "TOPRIGHT",-48,-5)
	report:SetScript("OnClick", function(f) HealingTracker.ReportFrame:Show() end)
	report:SetScript("OnEnter",
	function(self, button)
		GameTooltip:SetOwner(report, "ANCHOR_NONE")
		GameTooltip:SetPoint("BOTTOMRIGHT", report, "TOPLEFT")
		GameTooltip:ClearLines()
		GameTooltip:AddLine("Report", 0.8, 0.8, 0.8)
		GameTooltip:Show()
	end)
	report:SetScript("OnLeave", function(self, button) GameTooltip:Hide() end)

	local sett = CreateFrame("Button", nil, titleBar)
	sett:SetNormalTexture("Interface\\AddOns\\HealingTracker\\textures\\options")
	sett:SetHighlightTexture("Interface\\AddOns\\HealingTracker\\textures\\options", "ADD")
	sett:SetSize(12, 12)
	sett:SetPoint("TOPRIGHT", window, "TOPRIGHT",-33,-5)
	sett:SetScript("OnClick", function(f) HealingTracker.OptionsFrame:Show() end)
	sett:SetScript("OnEnter",
	function(self, button)
		GameTooltip:SetOwner(sett, "ANCHOR_NONE")
		GameTooltip:SetPoint("BOTTOMRIGHT", sett, "TOPLEFT")
		GameTooltip:ClearLines()
		GameTooltip:AddLine("Options", 0.8, 0.8, 0.8)
		GameTooltip:Show()
	end)
	sett:SetScript("OnLeave", function(self, button) GameTooltip:Hide() end)
	
	local reset = CreateFrame("Button", nil, titleBar)
	reset:SetNormalTexture("Interface\\AddOns\\HealingTracker\\textures\\reset")
	reset:SetHighlightTexture("Interface\\AddOns\\HealingTracker\\textures\\reset", "ADD")
	reset:SetSize(12, 12)
	reset:SetPoint("TOPRIGHT", window, "TOPRIGHT",-19,-5)
	reset:SetScript("OnClick", function(f)
		local rmenu = CreateFrame("Frame", "Reset")
		rmenu.displayMode = "MENU"
		rmenu.initialize = function(self, level)
			local cur_btn = {
				text = "Remove Current",
				func = function() HealingTracker:Reset(2) end
			}
			UIDropDownMenu_AddButton(cur_btn, level)

			local trash_btn = {
				text = "Reset Trash Data",
				func = function() HealingTracker:Reset(1) end
			}
			UIDropDownMenu_AddButton(trash_btn, level)

			local all_btn = {
				text = "Reset All Data",
				func = function() HealingTracker:Reset(0) end
			}
			UIDropDownMenu_AddButton(all_btn, level)
		end
		local x,y = GetCursorPosition(UIParent);
		ToggleDropDownMenu(1, nil, rmenu, "UIParent", x / UIParent:GetEffectiveScale() , y / UIParent:GetEffectiveScale())
	end)
	reset:SetScript("OnEnter",
	function(self, button)
		GameTooltip:SetOwner(reset, "ANCHOR_NONE")
		GameTooltip:SetPoint("BOTTOMRIGHT", reset, "TOPLEFT")
		GameTooltip:ClearLines()
		GameTooltip:AddLine("Reset Data", 0.8, 0.8, 0.8)
		GameTooltip:Show()
	end)
	reset:SetScript("OnLeave", function(self, button) GameTooltip:Hide() end)

	local close = CreateFrame("Button", nil, titleBar)
	close:SetNormalTexture("Interface\\AddOns\\HealingTracker\\textures\\close")
	close:SetHighlightTexture("Interface\\AddOns\\HealingTracker\\textures\\close", "ADD")
	close:SetSize(19, 19)
	close:SetPoint("TOPRIGHT", window, "TOPRIGHT",0,-2)
	close:SetScript("OnClick", function(f) 
		pdb.WindowShown = false
		TrackerFrame:Hide()
	end)
	close:SetScript("OnEnter",
	function(self, button)
		GameTooltip:SetOwner(close, "ANCHOR_NONE")
		GameTooltip:SetPoint("BOTTOMRIGHT", close, "TOPLEFT")
		GameTooltip:ClearLines()
		GameTooltip:AddLine("Hide Healing Tracker", 0.8, 0.8, 0.8)
		GameTooltip:Show()
	end)
	close:SetScript("OnLeave", function(self, button) GameTooltip:Hide() end)

	window.ReportButton = report
	window.CloseButton = close
	window.ResetButton = reset
	window.OptionsButton = sett

	tbar = CreateFrame("Frame", nil, titleBar)
	tbar:SetWidth(80)
	tbar:SetHeight(25)
	tbar:SetPoint("TOPLEFT", window, "TOPLEFT")
    tbar.Backdrop = CreateFrame("Frame", "tbarBackdrop", titleBar, "BackdropTemplate")
    tbar.Backdrop:SetAllPoints()
    tbar.Backdrop.backdropInfo = {
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark"
    }
    tbar.Backdrop:ApplyBackdrop()
    tbar.Backdrop:SetBackdropColor(0, 1, 0.35)

	tbar.Text1 = tbar:CreateFontString(nil, nil, "GameFontNormalSmall")
	tbar.Text1:SetPoint("LEFT", tbar, "LEFT", 4, 2)
	tbar.Text1:SetJustifyH("LEFT")

	tbar.Text1:SetFont(LSM:Fetch("font", pdb.Bars.Font), 9, "")
	tbar.Text1:SetTextColor(1, 1, 1, 1)
	tbar.Text1:SetWidth(window:GetWidth())
	tbar.Text1:SetHeight(20)
	tbar.Text1:SetNonSpaceWrap(false)
	tbar.Text1:SetText("Healing Tracker")
	tbar:SetScript("OnEnter",
	function(self, button)
		GameTooltip:SetOwner(tbar, "ANCHOR_NONE")
		GameTooltip:SetPoint("TOPLEFT", tbar, "TOPLEFT", 0, 35)
		GameTooltip:ClearLines()
		GameTooltip:AddLine("Right click to list segments", 0.8, 0.8, 0.8)
		GameTooltip:Show()
	end)
	tbar:SetScript("OnLeave", function(self, button) GameTooltip:Hide() end)
	window.tbar = tbar

	tbar:SetScript("OnMouseDown",
					function(self, button)
						if button == "LeftButton" then
							window.isMoving = 1
							window:StartMoving();
						elseif button == "RightButton" then
                            HealingTracker.Segments:ShowMenu()
                            TrackerFrame:StopMoving()
						end
					end)

	tbar:SetScript("OnMouseUp",
                    function(self)
                        TrackerFrame:StopMoving()
                    end)
                    
	tbar.Backdrop:SetScript("OnMouseDown",
					function(self, button)
						if button == "LeftButton" then
							window.isMoving = 1
							window:StartMoving();
						elseif button == "RightButton" then
                            HealingTracker.Segments:ShowMenu()
                            TrackerFrame:StopMoving()
						end
					end)

	tbar.Backdrop:SetScript("OnMouseUp",
                    function(self)
                        TrackerFrame:StopMoving()
					end)

	local bodyScrollFrame = CreateFrame("ScrollFrame", nil, window)
	bodyScrollFrame:SetWidth(window:GetWidth())
	bodyScrollFrame:SetHeight(window:GetHeight()-20)
	bodyScrollFrame:SetPoint("TOPLEFT", 0, -20)
	bodyScrollFrame:EnableMouseWheel(true)
	window.bodyScrollFrame = bodyScrollFrame

	local bodyFrame = CreateFrame("ScrollFrame", nil, window)
	bodyFrame:SetWidth(window:GetWidth())
	bodyFrame:SetHeight(window:GetHeight()-20)
	bodyFrame:SetPoint("TOPLEFT", 0, 0)
	window.bodyFrame = bodyFrame
	bodyScrollFrame:SetScrollChild(bodyFrame)

	bodyScrollFrame:SetScript("OnMouseWheel", function(self, delta)
		local diff = pdb.Bars.Height
		if delta > 0 then
			diff = diff * - 1
		end

		local nval = bodyScrollFrame:GetVerticalScroll() + diff
		if nval < 0 then nval = 0 end

		local pcount = #HealingTracker.Segments.Current.Players
		if nval > ((pcount - 2) * pdb.Bars.Height) then nval = (pcount - 2) * pdb.Bars.Height end
		if nval < 0 then nval = 0 end

		bodyScrollFrame:SetVerticalScroll(nval)
	end)

	bodyScrollFrame:SetVerticalScroll(0)

	window.DragBottomRight = CreateFrame("Button", nil, window)
	window.DragBottomRight:Show()
	window.DragBottomRight:SetFrameLevel(window:GetFrameLevel() + 10)
	window.DragBottomRight:SetNormalTexture("Interface\\AddOns\\HealingTracker\\textures\\sizing")
	window.DragBottomRight:SetHighlightTexture("Interface\\AddOns\\HealingTracker\\textures\\sizing")
	window.DragBottomRight:SetWidth(16)
	window.DragBottomRight:SetHeight(16)
	window.DragBottomRight:SetPoint("BOTTOMRIGHT", window, "BOTTOMRIGHT", 0, 0)
	window.DragBottomRight:EnableMouse(true)
	window.DragBottomRight:SetScript("OnMouseDown",
		function(self,button)
			if button == "LeftButton" then
				self:GetParent().isResizing = true;
				self:GetParent():StartSizing("BOTTOMRIGHT")
			end
		end)

	window.DragBottomRight:SetScript("OnMouseUp",
		function(self,button)
			if self:GetParent().isResizing == true then
				self:GetParent():StopMovingOrSizing()
				self:GetParent().isResizing = false
				TrackerFrame:FinishMoving()
				TrackerFrame:ApplyResizing()
			end
		end)

end

-- show frame
function TrackerFrame:Show()
	if window then
		window:Show()
		pdb.WindowShown = true
		
		if HealingTracker.Segments.Current ~= nil then
			TrackerFrame:UpdateBars()
		end

		for i=1,40 do
			local bar = TrackerFrame:CreateBar(i)
			windowBars[i] = bar
		end

		HealingTracker.Segments:Navigate(pdb.CurrentSegment)
		TrackerFrame:Redraw()
	end
end

function TrackerFrame:Hide()
	window:Hide()
end

function TrackerFrame:Reset()
end

function TrackerFrame:ResetScrollPos()
	window.bodyScrollFrame:SetVerticalScroll(0)
end

function TrackerFrame:SetSegment(segment)
	window.bodyScrollFrame:SetVerticalScroll(0)
	TrackerFrame:UpdateBars()
end

function TrackerFrame:Refresh()
end

function TrackerFrame:Resize(width, height)
end

function TrackerFrame:Redraw()
	if not pdb then return end

	local rx = -5
	if pdb.HeaderButtons.Close then
		window.CloseButton:Show()
		rx = -19
	else
		window.CloseButton:Hide()
	end

	if pdb.HeaderButtons.Reset then
		window.ResetButton:Show()
		window.ResetButton:SetPoint("TOPRIGHT", window, "TOPRIGHT",rx,-5)
		rx = rx - 14
	else
		window.ResetButton:Hide()
	end

	if pdb.HeaderButtons.Options then
		window.OptionsButton:Show()
		window.OptionsButton:SetPoint("TOPRIGHT", window, "TOPRIGHT",rx,-5)
		rx = rx - 15
	else
		window.OptionsButton:Hide()
	end

	if pdb.HeaderButtons.Report then
		window.ReportButton:Show()
		window.ReportButton:SetPoint("TOPRIGHT", window, "TOPRIGHT",rx,-5)
	else
		window.ReportButton:Hide()
	end

	window.tbar.Text1:SetFont(LSM:Fetch("font", pdb.Bars.Font), 9, "")

	table.foreach(windowBars,
	function(k,v)
		v:SetHeight(pdb.Bars.Height)
		v.Icon:SetWidth(pdb.Bars.Height)
		v.Icon:SetHeight(pdb.Bars.Height)
		v.BackLine:SetHeight(pdb.Bars.Height)
		v.txtPlayer:SetPoint("LEFT", v, "LEFT", pdb.Bars.Height + 6, -1)
		v.txtPlayer:SetWidth(window:GetWidth() - 95)

		v.BackLine.Backdrop.backdropInfo = {bgFile = LSM:Fetch("statusbar", pdb.Bars.Texture) }
		v.txtPlayer:SetFont(LSM:Fetch("font", pdb.Bars.Font), pdb.Bars.FontSize, "")

		local cl = pdb.Bars.FontColor
		v.txtPlayer:SetTextColor(cl.R, cl.G, cl.B, 1)
		v.txtPoint:SetFont(LSM:Fetch("font", pdb.Bars.Font), pdb.Bars.FontSize, "")

		v.txtPoint:SetTextColor(cl.R, cl.G, cl.B, 1)
		v.txtHeal:SetFont(LSM:Fetch("font", pdb.Bars.Font), pdb.Bars.FontSize, "")

        v.txtHeal:SetTextColor(cl.R, cl.G, cl.B, 1)
        local size = ((window:GetWidth() - pdb.Bars.Height) * v.SizeRatio) + pdb.Bars.Height
        if size > 0 then size = size - 1 end
        v.BackLine.Backdrop:SetWidth(size - 1)
		if v.Player ~= nil then
			local c = HealingTracker.Constants.ClassColors[v.Player.Class]
			if c then
				v.BackLine.Backdrop:SetBackdropColor(c.R, c.G, c.B, 0.8)
			end
		end
	end)

	window.tbar.Text1:SetFont(LSM:Fetch("font", pdb.Bars.Font), 9, "")

end

function TrackerFrame:UpdateBars()
	if not pdb then return end
	local selectedSegment = HealingTracker.Segments:SortCurrent()
	local ph = pdb.Bars.Height
	local fsize = window:GetWidth() - ph
	local maxPoint = -1
	local num = 1
	for i, v in ipairs(selectedSegment.Players) do
        local got = v ~= nil
		if v ~= nil and (pdb.Columns.Right < 5 or v.IsHealer == 1) then
			local vpoint = TrackerFrame:GetBarPoint(v, pdb.Columns.Right, pdb.Columns.RightHPS)
			if maxPoint < vpoint then
				maxPoint = vpoint
			end
			local bar = windowBars[num]
			if bar == nil then break end
			local vother = nil
			if pdb.Columns.Mid > 0 then
				vother = TrackerFrame:GetBarPoint(v, pdb.Columns.Mid, pdb.Columns.MidHPS)
			end
			if vpoint > 0 or (v.Cooldowns ~= nil and v.Cooldowns > 0) or (vother ~= nil and vother > 0) then
				bar.SizeRatio = vpoint / maxPoint
				local c = HealingTracker.Constants.ClassColors[v.Class]
				if c then
					bar.BackLine.Backdrop:SetBackdropColor(c.R, c.G, c.B, 0.8)
					local t = HealingTracker.Constants.ClassTextures[v.Class]
					bar.Icon.Tex:SetTexture("Interface\\AddOns\\HealingTracker\\textures\\"..t)
				end
				local bw = (fsize * bar.SizeRatio) + ph
                bar.BackLine:SetWidth(bw - 1)
                bar.BackLine.Backdrop:SetWidth(bw - 1)
				if pdb.Columns.Right == 5 or pdb.Columns.Right == 6 then
					bar.txtPoint:SetText(HealingTracker.Helper:ReadableNumber(vpoint).."% ")
                elseif pdb.Columns.Right == 1 then
					bar.txtPoint:SetText(HealingTracker.Helper:ReadableNumber(vpoint).." pts")
				else
					bar.txtPoint:SetText(HealingTracker.Helper:ReadableNumber(vpoint).." ")
				end
				if vother then
					if pdb.Columns.Mid == 5 or pdb.Columns.Mid == 6 then
						bar.txtHeal:SetText(HealingTracker.Helper:ReadableNumber(vother).."%")
                    elseif pdb.Columns.Mid == 1 then
						bar.txtHeal:SetText(HealingTracker.Helper:ReadableNumber(vother).." pts")
					else
						bar.txtHeal:SetText(HealingTracker.Helper:ReadableNumber(vother))
					end
				else
					bar.txtHeal:SetText(" ")
				end
                bar.Player = v
				bar.txtPlayer:SetText(num..". "..v.Name)

				num = num + 1
				bar:Show()
				bar.BackLine:Show()
			end
		end
	end

	visibleBarCount = num

	for i=num, 40 do
		windowBars[i]:Hide()
		windowBars[i].BackLine:Hide()
	end

end

function TrackerFrame:CheckBarBound(bar, barIndex)
	local boundsTop = window.bodyScrollFrame:GetVerticalScroll()
	local boundsBottom = boundsTop + pdb.Bounds.Height - 20
	local point, relativeTo, relativePoint, xOffset, yOffset = bar:GetPoint()
	local btop = yOffset * -1
	if btop + 3 > boundsTop and btop + pdb.Bars.Height - 3 < boundsBottom then
		return 1
	end
	return nil
end

function TrackerFrame:CreateBar(barIndex)
	if not pdb then return end
    local bar = CreateFrame("FRAME", "BarNo"..barIndex, window.bodyFrame)

	bar:SetWidth(window.bodyFrame:GetWidth())
	bar:SetHeight(pdb.Bars.Height)
	bar:SetPoint("LEFT", window.bodyFrame, 0)
	local topval = -(barIndex - 1) * pdb.Bars.Height
    bar:SetPoint("TOP", window.bodyFrame, "TOP", 0, topval)

	bar:SetScript("OnMouseDown",
		function(self,button)
			if TrackerFrame:CheckBarBound(bar, barIndex) then
				HealingTracker.DetailsFrame:Show(bar.Player)
			end
        end)

	bar:SetScript("OnEnter",
		function(self, button)
			if TrackerFrame:CheckBarBound(bar, barIndex) then
				bar.TooltipShown = 1
				GameTooltip:SetOwner(bar, "ANCHOR_NONE")
				GameTooltip:SetPoint("TOPLEFT", bar, "BOTTOMRIGHT")
				GameTooltip:ClearLines()
				GameTooltip:AddLine((bar.Player.TotalMultiply + bar.Player.TotalRegular).." point(s)", 0, 1, 0)
				GameTooltip:AddLine(bar.Player.TotalMultiply.." emergency point(s)", 1, 1, 1)
				GameTooltip:AddLine(bar.Player.TotalRegular.." regular point(s)", 1, 1, 1)
				GameTooltip:AddLine(HealingTracker.Helper:ReadableNumber(bar.Player.AidfulHeal + bar.Player.AidfulPeriodicHeal).." aidful heal", 1, 1, 1)
				GameTooltip:AddLine(HealingTracker.Helper:ReadableNumber(bar.Player.PointlessHeal + bar.Player.PointlessPeriodicHeal).." pointless heal", 1, 1, 1)
				GameTooltip:AddLine(HealingTracker.Helper:ReadableNumber(bar.Player.AidfulHeal + bar.Player.AidfulPeriodicHeal + bar.Player.PointlessHeal + bar.Player.PointlessPeriodicHeal).." total heal", 1, 1, 1)
				GameTooltip:AddLine(HealingTracker.Helper:ReadableNumber(bar.Player.OverHeal).." over heal", 1, 1, 1)
				GameTooltip:AddLine("Click to see details", 0.5, 0.5, 0.5)
				GameTooltip:Show()
			end
		end)
	bar:SetScript("OnLeave",
		function(self, button)
			if bar.TooltipShown then
				GameTooltip:Hide()
				bar.TooltipShown = nil
			end
		end)

	bar.BackLine = CreateFrame("FRAME", "BarLine"..barIndex, window.bodyFrame)
	bar.BackLine:SetPoint("TOPLEFT", bar, "TOPLEFT", 1, 0)
	bar.BackLine:SetHeight(bar:GetHeight())
	bar.BackLine:SetWidth(window:GetWidth() - 1)

    bar.BackLine.Backdrop = CreateFrame("Frame", "barBackline"..barIndex, bar, "BackdropTemplate")
    bar.BackLine.Backdrop:SetAllPoints(bar.BackLine)
    bar.BackLine.Backdrop.backdropInfo = { bgFile = LSM:Fetch("statusbar", pdb.Bars.Texture) }
    bar.BackLine.Backdrop:ApplyBackdrop()
	bar.BackLine.Backdrop:SetBackdropColor(0, 0, 0, 0.01)

	bar.TextBar = CreateFrame("FRAME", "TextBar"..barIndex, bar)
    bar.TextBar:SetAllPoints()

	local cl = pdb.Bars.FontColor
	bar.txtPlayer = bar.TextBar:CreateFontString(nil, bar.TextBar, "GameFontNormalSmall")
	bar.txtPlayer:SetPoint("LEFT", bar.TextBar, "LEFT", pdb.Bars.Height + 6, -1)
	bar.txtPlayer:SetJustifyH("LEFT")
	bar.txtPlayer:SetFont(LSM:Fetch("font", pdb.Bars.Font), pdb.Bars.FontSize, "")
    bar.txtPlayer:SetTextColor(cl.R, cl.G, cl.B, cl.A)
	bar.txtPlayer:SetWidth(window:GetWidth() - 95)
	bar.txtPlayer:SetNonSpaceWrap(false)
	bar.txtPlayer:SetText("")
	bar.txtPlayer:SetShadowColor(0,0,0,1)
    bar.txtPlayer:SetShadowOffset(1, -1)

	bar.Icon = CreateFrame("FRAME", "BarIcon"..barIndex, bar)
	bar.Icon:SetWidth(pdb.Bars.Height)
	bar.Icon:SetHeight(pdb.Bars.Height)
    bar.Icon:SetPoint("LEFT", 1, 0)

	local tex = bar.Icon:CreateTexture()
	tex:SetAllPoints(bar.Icon)
	tex:SetTexture("Interface\\AddOns\\HealingTracker\\textures\\shaman")
	bar.Icon.Tex = tex

	bar.txtPoint = bar.TextBar:CreateFontString(nil, bar.TextBar, "GameFontNormalSmall")
	bar.txtPoint:SetPoint("RIGHT", bar.TextBar, "RIGHT", -4, -1)
	bar.txtPoint:SetJustifyH("RIGHT")
	bar.txtPoint:SetFont(LSM:Fetch("font", pdb.Bars.Font), pdb.Bars.FontSize, "")
	bar.txtPoint:SetTextColor(cl.R, cl.G, cl.B, cl.A)
	bar.txtPoint:SetWidth(50)
	bar.txtPoint:SetHeight(pdb.Bars.Height)
	bar.txtPoint:SetNonSpaceWrap(false)
	bar.txtPoint:SetText("")
	bar.txtPoint:SetShadowColor(0,0,0,1)
	bar.txtPoint:SetShadowOffset(1, -1)

	bar.txtHeal = bar.TextBar:CreateFontString(nil, bar.TextBar, "GameFontNormalSmall")
	bar.txtHeal:SetPoint("RIGHT", bar.TextBar, "RIGHT", -50, -1)
	bar.txtHeal:SetJustifyH("RIGHT")
	bar.txtHeal:SetFont(LSM:Fetch("font", pdb.Bars.Font), pdb.Bars.FontSize, "")
	bar.txtHeal:SetTextColor(cl.R, cl.G, cl.B, cl.A)
	bar.txtHeal:SetWidth(80)
	bar.txtHeal:SetHeight(pdb.Bars.Height)
	bar.txtHeal:SetNonSpaceWrap(false)
	bar.txtHeal:SetText("")
	bar.txtHeal:SetShadowColor(0,0,0,1)
	bar.txtHeal:SetShadowOffset(1, -1)

	bar.SizeRatio = 1

	bar:Hide()
	bar.BackLine:Hide()

	return bar
end

function TrackerFrame:ApplyResizing()
	local ph = pdb.Bars.Height
	table.foreach(windowBars,
    function(k,v)
		v.txtPlayer:SetWidth(window:GetWidth() - 95)
        v:SetWidth(window:GetWidth() - 1)
        local size = ((window:GetWidth() - ph) * v.SizeRatio) + ph
        if size > 0 then size = size - 1 end
		v.BackLine:SetWidth(size)
	end)

	window.bodyScrollFrame:SetHeight(window:GetHeight()-20)
	window.bodyFrame:SetHeight(window:GetHeight()-20)
end

function TrackerFrame:FinishMoving()
	local pdb = HealingTracker.ProfileOptions
	local point, relativeTo, relativePoint, xOfs, yOfs = window:GetPoint()
	pdb.Bounds.XOff = xOfs
	pdb.Bounds.YOff = yOfs
	pdb.Bounds.Point = point
	pdb.Bounds.RelativePoint = relativePoint
	pdb.Bounds.Width = window:GetWidth()
	pdb.Bounds.Height = window:GetHeight()
end

function TrackerFrame:UpdateHeights()
	if not pdb then return end
	local ph = pdb.Bars.Height
	local i = 1
	table.foreach(windowBars,
	function(k,v)
		v.txtPlayer:SetWidth(window:GetWidth() - 95)
		v:SetHeight(pdb.Bars.Height)
        v:SetWidth(window:GetWidth())
        local size = ((window:GetWidth() - ph) * v.SizeRatio) + ph
        if size > 0 then size = size - 1 end
		v.BackLine:SetWidth(size - 1)
		v:SetPoint("TOP", window.bodyFrame, "TOP", 0, -(i - 1) * pdb.Bars.Height)
		i = i + 1
	end)
end

function TrackerFrame:GetBarPoint(v, t, hps)
	-- point
	if t == 1 then
		return v.TotalMultiply + v.TotalRegular

	-- total heal
	elseif t == 2 then
		if hps then
			return (v.AidfulHeal + v.PointlessHeal + v.AidfulPeriodicHeal + v.PointlessPeriodicHeal) / HealingTracker.Segments.Current.Duration
		else
			return v.AidfulHeal + v.PointlessHeal + v.AidfulPeriodicHeal + v.PointlessPeriodicHeal
		end

	-- aidful heal
	elseif t == 3 then
		if hps then
			return (v.AidfulHeal + v.AidfulPeriodicHeal) / HealingTracker.Segments.Current.Duration
		else
			return v.AidfulHeal + v.AidfulPeriodicHeal
		end

	-- pointless heal
	elseif t == 4 then
		if hps then
			return (v.PointlessHeal + v.PointlessPeriodicHeal) / HealingTracker.Segments.Current.Duration
		else
			return v.PointlessHeal + v.PointlessPeriodicHeal
		end

	-- mana
	elseif t == 5 then
		if v.CurrentMana then
			return v.CurrentMana
		else
			return 0
		end
	
	-- losing mana
	elseif t == 6 then
		local secs = #v.Mana
		local firstMana = 0
		local curMana = v.CurrentMana
		if secs > pdb.ManaThreshold then
			firstMana = v.Mana[secs - pdb.ManaThreshold]
		elseif secs > 1 then
			firstMana = v.Mana[1]
		else
			v.LoseMana = 0
			return v.LoseMana
		end
		if firstMana < curMana then
			v.LoseMana = 0
			return v.LoseMana
		else
			v.LoseMana = firstMana - curMana
			return v.LoseMana
		end
		
	-- active heal
	elseif t == 7 then
		if v.ActiveHeal then
			if hps then
				if v.ActiveHeal < 1 then
					return 0
				end
				return v.ActiveHeal / pdb.ActiveHealThreshold
			else
				return v.ActiveHeal
			end
		else
			return 0
		end
	end
	
	return 0
end

function TrackerFrame:StopMoving()
    if window.isMoving == 1 then
        window:StopMovingOrSizing()
        window.bodyFrame:SetHeight(window:GetHeight() - 20)
        TrackerFrame:FinishMoving()
    end
end